/***************************************************************************
 *   Copyright (C) 2005-2007 by David Charypar   *
 *   david.charypar@switzerland.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ALLSIM_MICROSIM_AGENT_H
#define ALLSIM_MICROSIM_AGENT_H

#include "callable.h"
//#include "world.h"
#include "clock.h"
#include "trip.h"
#include <vector>

using namespace std;

namespace allsim {
namespace microsim {

class World;
class RoadStart;
class RoadEnd;

/**
@author David Charypar
*/
class Agent : public Callable
{
public:
	Agent(unsigned id, World &theWorld);
	virtual ~Agent();
	//methods for setting up an agent
	unsigned getId() const;
	void deleteAllTrips();
	void addTrip(Trip *trip);
	void addActivityType(char actType);
	void deduceEndLinksOfTrips();

	// behavioral stuff
	void handleTimeToEnterRoad(double time);
	void handleEarliestTimeToLeaveRoad(double time);
	void handleTimeToStartTrip(double time);
	virtual void timerExpired();
	virtual int getSortPriority() const;
	virtual string getTypeString() const {return string("Agent");};

	bool getSqueezeFlag() const;
	void setSqueezeFlag(bool newState);
	unsigned getCurrentTripIndex();
	unsigned getFirstLinkId() const;
	double getLastRegisterTime() const;
	char getActivityType(unsigned index) const;
	Trip const *getTrip(unsigned index) const;
	void startActing();
	void serializeToVector(vector<char> *theVector) const;
	vector<char>::const_iterator unserializeFromVectorIterator(vector<char>::const_iterator iter);
	unsigned getNumTrips() const;
	unsigned getTotalLengthOfRoutes() const;
	//debugging stuff
	static unsigned getNumCreatedAgents();
	bool isDeleted();

private:
	Agent(Agent const &other);
	void computeNextRoad();
	void processNextTrip();

	bool deleted;
	unsigned const id;
	World &myWorld;
	Clock &myClock;
	unsigned currentRoadID, nextRoadID;
	RoadEnd *currentRoadEnd;
	RoadStart *nextRoadStart;
	double freeSpeedLeavingTime;
	double timerTime;
	double nextTimerTime;
	unsigned nextTimerType;
	double lastRoadRegisterTime;
	vector<Trip*> trips;
	vector<char> actTypes;
	unsigned currentTripIndex;
	bool squeezeFlag;
	
	// constants for recogintion of the type of a timer
	static unsigned const ENTER_LINK;
	static unsigned const LEAVE_LINK;
	static unsigned const START_TRIP;
	static unsigned creationCounter;
};

}
}

#endif
