/***************************************************************************
 *   Copyright (C) 2005-2007 by David Charypar   *
 *   david.charypar@switzerland.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "agentsreader.h"
#include "network.h"
#include "world.h"
#include "simnode.h"
#include "link.h"
#include "agent.h"
#include "trip.h"
#include "communicator.h"
#include <cassert>
#include <expat.h>
#include <iostream>
#include <fstream>
#include <algorithm>
#include <cstring>
#include <sstream>
//#include <arpa/inet.h>
//#include <wininet.h>

using namespace std;

namespace allsim 
{
namespace microsim
{
//using namespace tools;

// static member variables...
unsigned AgentsReader::depth;
char AgentsReader::Buff[BUFFSIZE];

Agent *AgentsReader::currentAgent = NULL;
World *AgentsReader::theWorld = NULL;



bool AgentsReader::isInsideRoute = false;
bool AgentsReader::currentPlanSelected = false;

char AgentsReader::routeBuffer[BUFFSIZE];
unsigned int AgentsReader::routeBufferCounter;
unsigned int AgentsReader::currentActivityStartLinkId = numeric_limits<unsigned>::max();
Trip *AgentsReader::currentTrip = NULL;


void AgentsReader::loadAgentsFromXML(char const *filename, World *theWorld)
{
	AgentsReader::theWorld = theWorld;
	XML_Parser p = XML_ParserCreate(NULL);
	if (! p) {
		Communicator::getLogFile() << "Couldn't allocate memory for parser" << endl;
		exit(-1);
	}

	XML_SetElementHandler(p, AgentsReader::startHandler, AgentsReader::endHandler);
	XML_SetCharacterDataHandler(p, AgentsReader::characterDataHandler);

	if(DEBUG)
		Communicator::getLogFile() << "in AgentsReader::loadAgentsFromXML(): opening file " 
		     << filename << endl;

	ifstream xmlStream(filename);

	if(DEBUG)
	{
		if(!xmlStream.good())
			Communicator::getLogFile() << "opening file failed: " << filename << endl;
		else
			Communicator::getLogFile() << "opened file " << filename << endl;
	}	

	bool done = false;
	do 
	{
		int len;
		xmlStream.getline(Buff, BUFFSIZE);
		len = xmlStream.gcount();
		done = !xmlStream.good();

		if (! XML_Parse(p, Buff, max(len-1,0), done)) 
		{
			Communicator::getLogFile() << "Parse error at line " << XML_GetCurrentLineNumber(p) << ":" << endl;
			Communicator::getLogFile() << XML_ErrorString(XML_GetErrorCode(p))<< endl;
			abort();
		}
  	}
	while(!done);

	// freeing the XML-Parser
	XML_ParserFree(p);	
}

void AgentsReader::loadAgentsFromBinaryFile(char const *filename, World *theWorld)
{

	AgentsReader::theWorld = theWorld;
	if(DEBUG)
		Communicator::getLogFile() << "in AgentsReader::loadAgentsFromBinaryFile(): opening file " 
		     << filename << endl;

	ifstream binStream(filename, ifstream::binary|ifstream::in);

	if(DEBUG)
	{
		if(!binStream.good())
			Communicator::getLogFile() << "opening file failed: " << filename << endl;
		else
			Communicator::getLogFile() << "opened file " << filename << endl;
	}	

	bool done = false;
	long numPersons;
	cerr.precision(30);
	if(binStream.good())
	{
		// loop over all persons
		numPersons = Communicator::read32BitIntegerFromStream(binStream);
		if(DEBUG)
			cerr << "numPersons = " << numPersons << endl;
		for(long i = 0; binStream.good() && i < numPersons; ++i)
		{
			long personId = Communicator::read32BitIntegerFromStream(binStream);
			if(DEBUG)
				cerr << "personId = " << personId << endl;
			Agent *newBinAgent = new Agent(personId, *theWorld);
			long numLegs = Communicator::read32BitIntegerFromStream(binStream);
			if(DEBUG)
				cerr << "numLegs = " << numLegs << endl;
			// loop over all legs of a person
			for(long j = 0; binStream.good() && j < numLegs; ++j)
			{
				double departureTime = Communicator::readDoubleFromStream(binStream);
				if(DEBUG)
					cerr << "departure time = " << departureTime << endl;
				Trip *newBinTrip =  new Trip(departureTime, 0.0);
				newBinAgent->addTrip(newBinTrip);
				long numLinksInRoute = Communicator::read32BitIntegerFromStream(binStream);
				if(DEBUG)
					cerr << "num links in route = " << numLinksInRoute << endl;
				// loop over all links in the route of a leg
				for(long k = 0; binStream.good() && k < numLinksInRoute; ++k)
				{
					long linkID = Communicator::read32BitIntegerFromStream(binStream);
					if(DEBUG)
						cerr << "link-id = " << linkID << endl;
					
					newBinTrip->addLinkIdToBackOfRoute(linkID);
				}
			}
			theWorld->addAgent(newBinAgent);
		}
	}	
	if(!binStream.good())
		cerr << "Error: reached end of file during read in of binary population file" << endl;
}

void AgentsReader::startHandler(void *, const char *el, const char **attr) 
{
	// decide based on the tag string which handler method to call
	if(strcmp(el, "person")==0)
		agentHandlerStart(attr);
	else
	if(strcmp(el, "plan")==0)
		planHandlerStart(attr);
	else
	if(strcmp(el, "act")==0)
		actHandlerStart(attr);
	else
	if(strcmp(el, "leg")==0)
		legHandlerStart(attr);
	else
	if(strcmp(el, "route")==0)
		routeHandlerStart(attr);
	depth++;
}

void AgentsReader::endHandler(void *, const char *el) 
{
	depth--;
	
	// decide based on the tag string which handler method to call
	if(strcmp(el, "person")==0)
		agentHandlerEnd();
	else
	if(strcmp(el, "plan")==0)
		planHandlerEnd();
	else
	if(strcmp(el, "act")==0)
		actHandlerEnd();
	else
	if(strcmp(el, "leg")==0)
		legHandlerEnd();
	else
	if(strcmp(el, "route")==0)
		routeHandlerEnd();
}

void AgentsReader::characterDataHandler(void *, const char *s, int len)
{


	// this member function is for reading unformated data from the xml file.
	// this type of data is also called pc-data
	// for the time being we use this function only for reading in the routes from
	// the plans files...
	if(!isInsideRoute)
		return;
	// this does only copy the strings into one single buffer to make it available 
	// for processing the route as a whole.

	strncpy(routeBuffer+routeBufferCounter, s, len);
	routeBufferCounter += len;	
}

void AgentsReader::agentHandlerStart(const char **attr)
{
	unsigned int agentId = 0;
	for (int i = 0; attr[i]; i += 2)
	{
		if(strcmp(attr[i], "id")==0)
			agentId = atoi(attr[i+1]);
	}
	// create Agent here
	currentAgent = new Agent(agentId, *theWorld);
}

void AgentsReader::agentHandlerEnd()
{
	static unsigned counter = 0;

	counter++;
	if(counter % 100000 == 0)
		Communicator::getLogFile() << "read agent " << counter << "." << endl;


	// finish creation of Agent here 

	// check if this agent starts on a node that resides on this proc.
	// if it does, add the agent to the world else delete it
	if(theWorld->hasRoadStart(currentAgent->getFirstLinkId()))
	{
		if(counter % 1 == 0)
			theWorld->addAgent(currentAgent);
		else
			delete currentAgent;
	}
	else
	{	
		delete currentAgent;
		currentAgent = NULL;
	}
}

void AgentsReader::planHandlerStart(const char **attr)
{
	// in our model each agent has only one single plan. 
	// This means we have to find the selected plan 
	// and read only that one.
	

//	currentPlanSelected = false;
	// this is only for testing with old plans file format
	currentPlanSelected = true;
	for (int i = 0; attr[i]; i += 2)
	{
		if(strcmp(attr[i], "selected")==0)
		{
			if(strcmp(attr[i+1], "yes")==0)
			{
				currentPlanSelected = true;
			}
		}
	}
	
	
}

void AgentsReader::planHandlerEnd()
{
	// we have to do deduce the propper starting and end links to the trips
	// since they are missing in the plans file 

	currentAgent->deduceEndLinksOfTrips();
}

void AgentsReader::actHandlerStart(const char **attr)
{
	if(!currentPlanSelected)
		return;

	string activityType, activityStartTime, activityDuration;
	unsigned int activityRoadId;
	
	for (int i = 0; attr[i]; i += 2)
	{
		if(strcmp(attr[i], "type")==0)
			activityType = attr[i+1];
		else
		if(strcmp(attr[i], "link")==0)
		{
			activityRoadId = atoi(attr[i+1]);
			currentActivityStartLinkId = activityRoadId;
		}
		else
		if(strcmp(attr[i], "start_time")==0)
			activityStartTime = attr[i+1];
		else
		if(strcmp(attr[i], "dur")==0)
			activityDuration = attr[i+1];
	}
	currentAgent->addActivityType(activityType.c_str()[0]);
}

void AgentsReader::actHandlerEnd()
{
	if(!currentPlanSelected)
		return;
	// finish act here if necessary
}

void AgentsReader::legHandlerStart(const char **attr)
{
	// check if the plan this leg belongs to is selected
	// if not ignore the leg
	if(!currentPlanSelected)
		return;

	unsigned num;
	string travelMode, departureTimeString, travelTimeString;

	for (int i = 0; attr[i]; i += 2)
	{
		if(strcmp(attr[i], "num")==0)
			num = atoi(attr[i+1]);
		else
		if(strcmp(attr[i], "mode")==0)
			travelMode = attr[i+1];
		else
		if(strcmp(attr[i], "dep_time")==0)
			departureTimeString = attr[i+1];
		else
		if(strcmp(attr[i], "trav_time")==0)
			travelTimeString = attr[i+1];
	}
	double departureTime = convertTimeStringToTime(departureTimeString);
	double travelTime = convertTimeStringToTime(travelTimeString);
	// create the leg here...
	currentTrip = new Trip(departureTime, travelTime);
}

void AgentsReader::legHandlerEnd()
{
	if(!currentPlanSelected)
		return;

	static unsigned maxLength = 0;

	// finish creating leg here:
	// 1. add Trip to Agent
	currentAgent->addTrip(currentTrip);
	if(currentTrip->getSize() > maxLength)
	{
		maxLength = currentTrip->getSize();
//		cerr << "new max triplength = " << maxLength << " on agent " << currentAgent->getId() << endl;
	}
	
	// 2. Set "currentTrip"-pointer to NULL
	currentTrip = NULL;
}

void AgentsReader::routeHandlerStart(const char **)
{
	// check if the plan this route belongs to is selected
	// if not ignore the route
	if(!currentPlanSelected)
		return;


	// if we reach this point we know that some pcdata is going to follow. 
	// -> store the route in a vector
	isInsideRoute = true;
	routeBufferCounter = 0;
	
}

void AgentsReader::routeHandlerEnd()
{
	// check if the plan this route belongs to is selected
	// if not ignore the route
	if(!currentPlanSelected)
		return;


	// do the final creation of the route...

	// first: add the start link
	currentTrip->addLinkIdToBackOfRoute(currentActivityStartLinkId);
	// at this point all the pcdata from the route should have been stored in the
	// routeBuffer. Now we read in the nodes and create the trip from it.
	
	// make the string zero terminated
	routeBuffer[routeBufferCounter] = '\0';
	string tmp(routeBuffer);	
	istringstream inputRouteStream(tmp);
	Node *currentNode=NULL, *nextNode=NULL;
	if(inputRouteStream.good())
	{
		unsigned int nodeId;
		inputRouteStream >> nodeId;
//		cerr << "nodeId = " << nodeId << endl;
		currentNode = theWorld->getNetwork()->getNodeWithId(nodeId);
		while(inputRouteStream.good())
		{
			inputRouteStream >> nodeId;
//			cerr << "nodeId = " << nodeId << endl;
			nextNode = theWorld->getNetwork()->getNodeWithId(nodeId);
			Link *routeLink = currentNode->getLinkToNeighborNode(nextNode);
//			cerr << "link = " << routeLink->getId() << endl;
			currentTrip->addLinkIdToBackOfRoute(routeLink->getId());
			currentNode = nextNode;
		}
	}
	assert(currentTrip->getSize()>0);
		
	isInsideRoute = false;
}

}

}
