/***************************************************************************
 *   Copyright (C) 2005-2007 by David Charypar   *
 *   david.charypar@switzerland.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ALLSIMCOMMUNICATOR_H
#define ALLSIMCOMMUNICATOR_H

#include <vector>
#include <fstream>

using namespace std;

namespace allsim {
namespace microsim {


/**
@author David Charypar
*/
class Communicator
{
public:
	static void init(int &nArgs, char **&args);
	static void initCommon();
	static void close();
	static int getNumProcs();
	static int getProcId();
	static int getMaxmumTag();
	static bool isEventsFileBinary();
	static ofstream &getEventsFile();
	static ofstream &getComLogFile();
	static ofstream &getLogFile();
	static void sendToAllNeighbors(vector<char> &buffer, int messageTag);
	static void receiveFromAllNeighbors(vector<char> &buffer, int messageTag);
	static void sendToProcess(int procId, int messageTag, vector<char> &message, double time=-1.0);
	static void receiveFromProcess(int procId, int messageTag, vector<char> &message, double time=-1.0);
	static void waitForOtherProcesses();
	static void write32BitIntegerToStream(int32_t data, ostream &theStream);
	static void writeDoubleToStream(double data, ostream &theStream);
	static int32_t read32BitIntegerFromStream(istream &theStream);
	static double readDoubleFromStream(istream &theStream);


private:
	static int numProcs, myProcId;
	static ofstream eventsFile, comLogFile, logFile;
	static bool eventsFileIsBinary;
	static int byteOrder64[8];
	static int byteOrder32[4];
	static int byteOrder16[2];
};

}
}

#endif
