/***************************************************************************
 *   Copyright (C) 2005-2007 by David Charypar   *
 *   david.charypar@switzerland.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "configreader.h"
#include "microsimconfig.h"
#include <cassert>
#include <expat.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <new>

using namespace std;

namespace allsim 
{
namespace config
{
// static member variables...
char ConfigReader::Buff[BUFFSIZE];
string ConfigReader::moduleName, ConfigReader::paramName, ConfigReader::paramValue;



void ConfigReader::loadConfigFromXML(char const *filename)
{
	XML_Parser myXmlParser = XML_ParserCreate(NULL);
	if (myXmlParser == NULL) {
		cerr << "error during creation of XML parser" << endl;
		throw;
	}

	XML_SetElementHandler(myXmlParser, ConfigReader::startHandler, ConfigReader::endHandler);

	ifstream xmlStream(filename);

	bool done = false;
	while(!done) 
	{
		int len;
		xmlStream.getline(Buff, BUFFSIZE);
		len = xmlStream.gcount();
		done = !xmlStream.good();

		if (XML_Parse(myXmlParser, Buff, max(len-1,0), done) == false) 
		{
			cerr << "error parsing line " << XML_GetCurrentLineNumber(myXmlParser) << "." << endl;
			cerr << "error string: " << XML_ErrorString(XML_GetErrorCode(myXmlParser)) << endl;
			throw;
		}
  	}

	XML_ParserFree(myXmlParser);	
}

void ConfigReader::startHandler(void *, const char *element, const char **attributes) 
{
	// decide based on the tag string which handler method to call
	if(strcmp(element, "module")==0)
		moduleHandlerStart(attributes);
	else
		if(strcmp(element, "param")==0)
			paramHandlerStart(attributes);
}

void ConfigReader::endHandler(void *, const char *element) 
{
	// decide based on the tag string which handler method to call
	if(strcmp(element, "module")==0)
		moduleHandlerEnd();
	else
		if(strcmp(element, "param")==0)
			paramHandlerEnd();
}

// void ConfigReader::characterDataHandler(void *, const char *s, int len)
// {
// 
// 
// 	// this member function is for reading unformated data from the xml file.
// 	// this type of data is also called pc-data
// 	// for the time being we use this function only for reading in the routes from
// 	// the plans files...
// 	if(!isInsideRoute)
// 		return;
// 	// this does only copy the strings into one single buffer to make it available 
// 	// for processing the route as a whole.
// 
// 	strncpy(routeBuffer+routeBufferCounter, s, len);
// 	routeBufferCounter += len;	
// }

void ConfigReader::moduleHandlerStart(const char **attributes)
{
	bool hasModuleName = false;
	for (int i = 0; attributes[i]; i += 2)
	{
		if(strcmp(attributes[i], "name")==0)
		{
			moduleName = attributes[i+1];
			hasModuleName = true;
		}
	}
	assert(hasModuleName);
	Config::ensureModuleExists(moduleName);
}

void ConfigReader::moduleHandlerEnd()
{
}

void ConfigReader::paramHandlerStart(const char **attributes)
{
	bool hasParamName = false;
	bool hasParamValue = false;
	for (unsigned i = 0, counter = 0; attributes[i]; i += 2, ++counter)
	{
		if(strcmp(attributes[i], "name")==0)
		{
			paramName = attributes[i+1];
			if(DEBUG)
				cerr << "param name = " << paramName << endl;
			hasParamName = true;
		}
		else
			if(strcmp(attributes[i], "value")==0)
			{
				paramValue = attributes[i+1];
				hasParamValue = true;
				if(DEBUG)
					cerr << "attr value = " << paramValue << endl;
			}
	}
	assert(hasParamName);
	assert(hasParamValue);

	Config::getModuleConfig(moduleName).addParam(paramName, paramValue);
}

void ConfigReader::paramHandlerEnd()
{
}

}

}
