/***************************************************************************
 *   Copyright (C) 2005-2007 by David Charypar   *
 *   david.charypar@switzerland.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include "microsimconfig.h"
#endif

#include <iostream>
#include <cstdlib>
#include "agent.h"
#include "callable.h"
#include "trip.h"
#include "world.h"
#include "clock.h"
#include "simnetwork.h"
#include "networkreader.h"
#include "agentsreader.h"
#include "communicator.h"
#include "microsimconfig.h"
#include "configreader.h"
#include "stopwatch.h"
#include <unistd.h>



using namespace std;
using namespace allsim;
using namespace allsim::config;
using namespace allsim::microsim;



int main(int nArgs, char **args)
{
//	try
	{
	if(nArgs != 2 && nArgs != 6)
	{
		cerr << "usage: ./microsim <config-file.xml>" << endl;
		exit(EXIT_SUCCESS);
		Communicator::close();
	}

	tools::StopWatch overallWatch;
	tools::StopWatch basicInitWatch, networkReadWatch, networkPartitionWatch;
	tools::StopWatch roadCreationWatch, barrier1Watch, readAgentsWatch;
	tools::StopWatch barrier2Watch, initAgentsWatch, barrier3Watch, processTimersWatch;
	tools::StopWatch finalBarrierWatch, finalOutputWatch;

	overallWatch.start();
	basicInitWatch.start();

	// read in the configuration from the xml config file...
	config::ConfigReader::loadConfigFromXML(args[1]);
	// initialize communicator for communication with other processors
	Communicator::init(nArgs, args);
	
	ofstream &logFile = Communicator::getLogFile();

	// initialize world
	World theWorld;


	basicInitWatch.end();
	
	networkReadWatch.start();

//	Network *myNetwork = NetworkReader::loadNetworkFromXML("/home/davidch/cvs-berlin/vsp-cvs/studies/equil-test/network/equil.xml");
//	NetworkReader::loadNetworkFromXML("/home/davidch/cvs-berlin/vsp-cvs/studies/schweiz/2network/ch.xml");
	Network *theNetwork = NetworkReader::loadNetworkFromXML(
	                      Config::getModuleConfig(Config::NETWORK).getParam(Config::NETWORK_INPUTNETWORKFILE).c_str());
	logFile << "network contains " << theNetwork->getNumLinks() << " links" << endl;
	SimNetwork theSimNetwork(*theNetwork);
	delete theNetwork;
	if(DEBUG)
		logFile << "simnetwork contains " << theSimNetwork.getNumLinks() << " links" << endl;
	theWorld.setNetwork(&theSimNetwork);

	networkReadWatch.end();
	networkPartitionWatch.start();

	// preparation for splitting of network: read in loads file
	ifstream loadsInFile("loads_in.txt");
	if(loadsInFile)
	{
		theSimNetwork.loadLoadsFromStream(loadsInFile);
		loadsInFile.close();
	}
	

	// partition network and assign it the the processes.
	unsigned numBisectionSteps = 0;
	unsigned tmp = Communicator::getNumProcs();
	while(tmp > 1)
	{
		++numBisectionSteps;
		tmp >>= 1;
	}

	if(1<<numBisectionSteps != Communicator::getNumProcs())
	{
		logFile << "Warning! network can only be partitioned into a number of parts that is a power of 2" << endl;
		logFile << "Network is going to be partitioned into " << (1u<<numBisectionSteps) << " parts instead of " << Communicator::getNumProcs() << endl;
	}
	
	theSimNetwork.partitionNetwork(numBisectionSteps);
//	theSimNetwork.partitionNetwork(5);
	logFile << "The network was partitioned into " << (1u<<numBisectionSteps) << " parts." << endl;
	networkPartitionWatch.end();


// produce network patitioning output for visualization
	if(Communicator::getProcId() == 0)
	{ 	
		ofstream assingmentStream("linkprocs.txt");
	 	theSimNetwork.printLinkProcAssignmentToStream(assingmentStream);
	 	assingmentStream.close();
	}

	roadCreationWatch.start();
	theSimNetwork.createRoadsForWorld(theWorld);
	theWorld.directlyConnectInternalLinks();
	roadCreationWatch.end();

	barrier1Watch.start();
	Communicator::waitForOtherProcesses();
	barrier1Watch.end();

	readAgentsWatch.start();
	// select agents reader according to config file
	if(Config::getModuleConfig(Config::PLANS).getParam(Config::PLANS_INPUTVERSION).compare("matsimDEQv1") == 0)
	{
		cerr << "reading binary population file" << endl;
		AgentsReader::loadAgentsFromBinaryFile(Config::getModuleConfig(Config::PLANS).getParam(Config::PLANS_INPUTPLANSFILE).c_str(), &theWorld);
	}
	else
	{
		cerr << "reading xml pupulation file" << endl;
		AgentsReader::loadAgentsFromXML(Config::getModuleConfig(Config::PLANS).getParam(Config::PLANS_INPUTPLANSFILE).c_str(), &theWorld);
	}
	cerr << "done reading population file" << endl;
	readAgentsWatch.end();


	barrier2Watch.start();
	Communicator::waitForOtherProcesses();
	barrier2Watch.end();

	initAgentsWatch.start();
	if(DEBUG)
		logFile << "proc " << Communicator::getProcId() 
		     << ": num agents/roadStarts/roadEnds: "
		     << theWorld.countAgents() << "/"
		     << theWorld.countRoadStarts() << "/"
		     << theWorld.countRoadEnds() << endl;
	
	theWorld.startRoads();
	theWorld.startAgents();
	initAgentsWatch.end();

	barrier3Watch.start();
	Communicator::waitForOtherProcesses();
	barrier3Watch.end();

	processTimersWatch.start();	
	logFile << "--------- starting event loop " << endl;
	if(DEBUG)
		logFile << "at start, the clock contains " 
		     << theWorld.getClock().getNumTimersToDo() << " timers." << endl;
	theWorld.getClock().processTimers();
	logFile << "the system processed " << theWorld.getClock().getNumProcessedTimers() << " timers." << endl;
	processTimersWatch.end();

	finalBarrierWatch.start();
	Communicator::waitForOtherProcesses();
	finalBarrierWatch.end();

	finalOutputWatch.start();
	// write out loads of nodes for later load balancing:
	if(COMPUTE_LOADS)
	{
		ofstream loadsOutFile("loads_out.txt");
		theSimNetwork.printNodeLoadsToStream(loadsOutFile);
		loadsOutFile.close();
	}
	finalOutputWatch.end();
	overallWatch.end();


	logFile << "execution started at " << overallWatch.getStartTime();
	logFile << "execution ended at " << overallWatch.getEndTime();
	logFile << "total running time was " << overallWatch.getMeasuredTime() << " seconds." << endl;
	logFile << "statistics on time spent in different parts of program (in seconds): " << endl;
	logFile << basicInitWatch.getMeasuredTime() << ": basic initialization" << endl;
	logFile << networkReadWatch.getMeasuredTime() << ": reading of network" << endl;
	logFile << networkPartitionWatch.getMeasuredTime() << ": partitioning of network" << endl;
	logFile << roadCreationWatch.getMeasuredTime() << ": creation of roads" << endl;
	logFile << barrier1Watch.getMeasuredTime() << ": waiting for other procs before reading agents" << endl;
	logFile << readAgentsWatch.getMeasuredTime() << ": reading of agents" << endl;
	logFile << barrier2Watch.getMeasuredTime() << ": waiting for other procs after reading agents" << endl;
	logFile << initAgentsWatch.getMeasuredTime() << ": initialization of agents and roads" << endl;
	logFile << barrier3Watch.getMeasuredTime() << ": waiting for other procs before events loop" << endl;
	logFile << processTimersWatch.getMeasuredTime() << ": processing of timers" << endl;
	logFile << finalBarrierWatch.getMeasuredTime() << ": waiting for other procs after events loop" << endl;
	logFile << finalOutputWatch.getMeasuredTime() << ": producing loads file" << endl;
	
	Communicator::close();

 	}
//  	catch(char const * msg)
//  	{
//  		cerr << "exception message is: \"" << msg << "\"" << endl;
//  		throw;
//  	}


	return EXIT_SUCCESS;
}

