/***************************************************************************
 *   Copyright (C) 2005-2007 by David Charypar   *
 *   david.charypar@switzerland.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "microsimconfig.h"
#include <iostream>

namespace allsim {
namespace config {


map<string, ModuleConfig> Config::moduleConfigs;

// string constants according to xml
const string Config::SCENARIO("scenario");
const string Config::SIM("sim");
const string Config::DEQSIM("deqsim");
const string Config::EVENTS("events");
const string Config::GLOBAL("global");
const string Config::NETWORK("network");
const string Config::PLANS("plans");


//const string Config::SCENARIO_NETWORKFILENAME("networkFilename");
//const string Config::SCENARIO_INPUTPLANSFILENAME("inputPlansFilename");
//const string Config::SCENARIO_WORKINGPLANSFILENAME("workingPlansFilename");
//const string Config::SCENARIO_WORKINGEVENTSTXTFILENAME("workingEventsTxtFilename");
//const string Config::SCENARIO_WORIKINGSNAPSHOTFILENAME("workingSnapshotFilename");


// const string Config::SIM_STUCKTIME("stuckTime");
// const string Config::SIM_STARTTIME("sim_start_time");
// const string Config::SIM_CARSIZE("carSize");
// const string Config::SIM_FLOWCAPACITYFACTOR("flowCapacityFactor");
// const string Config::SIM_STORAGECAPACITYFACTOR("storageCapacityFactor");

const string Config::DEQSIM_STARTTIME("startTime");
const string Config::DEQSIM_ENDTIME("endTime");
const string Config::DEQSIM_SQUEEZETIME("squeezeTime");
const string Config::DEQSIM_GAPTRAVELSPEED("gapTravelSpeed");
const string Config::DEQSIM_CARSIZE("carSize");
const string Config::DEQSIM_FLOWCAPACITYFACTOR("flowCapacityFactor");
const string Config::DEQSIM_STORAGECAPACITYFACTOR("storageCapacityFactor");

const string Config::EVENTS_OUTPUTFILE("outputFile");
const string Config::EVENTS_OUTPUTFORMAT("outputFormat");

const string Config::NETWORK_INPUTNETWORKFILE("inputNetworkFile");

const string Config::PLANS_INPUTPLANSFILE("inputPlansFile");
const string Config::PLANS_INPUTVERSION("inputVersion");


void Config::ensureModuleExists(string moduleName)
{
	if(moduleConfigs.find(moduleName) == moduleConfigs.end())
	{
		moduleConfigs[moduleName] = ModuleConfig();
	}
}

ModuleConfig &Config::getModuleConfig(string moduleName)
{
	if(moduleConfigs.find(moduleName) == moduleConfigs.end())
	{
		cerr << "error: requested module " << moduleName << " does not exist" << endl;
		throw;
	}
	else
		return moduleConfigs.find(moduleName)->second;
}


void ModuleConfig::addParam(string paramName, string paramValue)
{
	params[paramName] = paramValue;
}

string ModuleConfig::getParam(string paramName) const
{
	if(params.find(paramName) != params.end())
		return params.find(paramName)->second;
	else
	{
		cerr << "error, trying to access a parameter " << paramName << " that does not exist..." << endl;
		throw;
	}
}

}
}
