/***************************************************************************
 *   Copyright (C) 2005-2007 by David Charypar   *
 *   david.charypar@switzerland.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ALLSIM_CONFIG_MICROSIMCONFIG_H
#define ALLSIM_CONFIG_MICROSIMCONFIG_H

#include <string>
#include <map>


using namespace std;

namespace allsim {
namespace config {

/**
	Holds the configuration parameters of the simulation.

	The class is usually not initialized directly. Instead, normally class ConfigReader should be used.
	@author David Charypar <david.charypar@switzerland.org>
*/

class ModuleConfig;

class Config
{
public:
	static void ensureModuleExists(string moduleName);
	static ModuleConfig &getModuleConfig(string moduleName);
	static const string SCENARIO;
	static const string SIM;
	static const string DEQSIM;
	static const string EVENTS;
	static const string GLOBAL;
	static const string NETWORK;
	static const string PLANS;


//	static const string SCENARIO_NETWORKFILENAME;
//	static const string SCENARIO_INPUTPLANSFILENAME;
//	static const string SCENARIO_WORKINGPLANSFILENAME;
//	static const string SCENARIO_WORKINGEVENTSTXTFILENAME;
	static const string SCENARIO_WORIKINGSNAPSHOTFILENAME;

	static const string SIM_STUCKTIME;
	static const string SIM_STARTTIME;
	static const string SIM_SQUEEZETIME;
	static const string SIM_GAPTRAVELSPEED;
	static const string SIM_CARSIZE;
	static const string SIM_FLOWCAPACITYFACTOR;
	static const string SIM_STORAGECAPACITYFACTOR;

	static const string DEQSIM_STARTTIME;
	static const string DEQSIM_ENDTIME;
	static const string DEQSIM_SQUEEZETIME;
	static const string DEQSIM_GAPTRAVELSPEED;
	static const string DEQSIM_CARSIZE;
	static const string DEQSIM_FLOWCAPACITYFACTOR;
	static const string DEQSIM_STORAGECAPACITYFACTOR;

	static const string EVENTS_OUTPUTFILE;
	static const string EVENTS_OUTPUTFORMAT;

	static const string NETWORK_INPUTNETWORKFILE;

	static const string PLANS_INPUTPLANSFILE;
	static const string PLANS_INPUTVERSION;


private:
	Config();
	~Config();
	static map<string, ModuleConfig> moduleConfigs;
};

class ModuleConfig
{
public:
	void addParam(string paramName, string paramValue);
	string getParam(string paramName) const;
private:
	map<string, string> params;
};

}
}

#endif
