/***************************************************************************
 *   Copyright (C) 2005-2007 by David Charypar   *
 *   david.charypar@switzerland.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ALLSIMROADEND_H
#define ALLSIMROADEND_H

#include "callable.h"
#include "world.h"
#include "network.h"
#include "clock.h"
#include <list>
#include <vector>

using namespace std;
namespace allsim {
namespace microsim {
class Agent;
class RoadStart;

/**
	@author David Charypar <david.charypar@switzerland.org>
*/
class RoadEnd : public Callable
{
public:
	// constructor, destructor
	RoadEnd(unsigned id, unsigned fromNodeId, unsigned toNodeId,
	        double freeSpeedTravelTime, double gapTravelTime, 
	        double minimalGap, unsigned sizeOfRoad, World &theWorld);
	virtual ~RoadEnd();

	// getters and setters
	unsigned getId();
	
	// methods for flow simulation
	void noteFirstAgentLeavesRoad(Agent *agent, double time);
	void checkTasks();
	void checkTasksFromPartner();
	virtual void timerExpired();
	virtual int getSortPriority() const;
	virtual string getTypeString() const {return string("RoadEnd");};

	void registerInitialTimerAtClock();


	// methods for message passing to other part of road
	void exchangeDataWithRoadStart();
	void exchangeDataWithRoadStartSend();
	void exchangeDataWithRoadStartReceive();

	// methods to create a direct connection to the corresponding road start
	bool isDirectlyConnectable();
	bool isDirectlyConnected();
	void directlyConnectToRoadStart();


	// debugging methods
	unsigned getNumberOfAgentsOnRoad() const;
	void printAgentOnRoad() const;

private:
//	methods only for internal use in flow simulation
	void notifyFirstCarAboutDepartureTime();

//	methods for data exchange with road end
	void createShortExchangeMessage();
	void createLongExchangeMessage();
	void emptyGapQueue();
	void sendExchangeMessage();
	void receiveExchangeMessage();
	void readShortExchangeMessage();
	void readLongExchangeMessage();


//	properties (= variables that usually should not change)
	unsigned id, fromNodeId, toNodeId;
	unsigned sizeOfRoad;
	double gapTravelTime;
	double syncTime;
	double minimalGap;
	World &myWorld;
	tools::Network &myNetwork;
	Clock &myClock;
// variables
	bool firstCarNotified;
	double lastLeavingTime;
	list<Agent*> *roadQueue;
	list<double> *gapQueue;
	double nextSyncTime;
	bool isInSync;
	bool directyConnected;
	RoadStart *partnerRoadStart;



//	message for data exchange with road end...
	vector<char> exchangeMessageSend;
	vector<char> exchangeMessageReceive;
};

}
}

#endif
