/***************************************************************************
 *   Copyright (C) 2005-2007 by David Charypar   *
 *   david.charypar@switzerland.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ALLSIMROADSTART_H
#define ALLSIMROADSTART_H

#include "callable.h"
#include "world.h"
#include "network.h"
#include "clock.h"
#include <list>
#include <vector>

using namespace std;
namespace allsim {
namespace microsim {

class Agent;
class SqueezeTimer;
class RoadEnd;



/**
	@author David Charypar <david.charypar@switzerland.org>
*/
class RoadStart : public Callable
{
public:
	// constructor, destructor
	RoadStart(unsigned id, unsigned fromNodeId, unsigned toNodeId, double freeSpeedTravelTime, double gapTravelTime, 
	          double minimalGap, unsigned sizeOfRoad,
	          double squeezeTime, World &theWorld);
	virtual ~RoadStart();

	// getters and setters
	unsigned getId();

	// methods for flow simulation
	void registerAgentForEntering(Agent *agent, double time);
	void registerAgentForStartingATrip(Agent *agent, double time);
	void noteAgentEntersRoad(Agent *agent, double time);
	void noteAgentEndsTrip(Agent *agent, double time);
	double getFreeSpeedLeavingTime(double entryTime) const;
	void checkTasks();
	void checkTasksFromPartner();
	virtual void timerExpired();
	virtual int getSortPriority() const;
	virtual string getTypeString() const {return string("RoadStart");};

	void registerInitialTimerAtClock();

	// methods for message passing to other part of road
	void exchangeDataWithRoadEnd();
	void exchangeDataWithRoadEndSend();
	void exchangeDataWithRoadEndReceive();

	// methods to create a direct connection to the corresponding road start
	list<Agent*> *getRoadQueue();
	list<double> *getGapQueue();
	void markDirectlyConnectedTo(RoadEnd *roadEnd);


	// debugging methods
	unsigned getNumberOfAgentsOnRoad() const;
	void printAgentOnRoad() const;

private:
//	methods only for internal use in flow simulation
	void findGapForFirstWaitingCar();
	void addSqueezeTimer(double time);
	double getNextSqueezeTimerTime();
	bool isNextSqueezeTimerValid();
	void removeNextSqueezeTimer();
	void invalidateNewestSqueezeTimer();

//	methods for data exchange with road end
	void createShortExchangeMessage();
	void createLongExchangeMessage();
	void deleteAgentsOnRoadFromQueueAndWorld();
	void sendExchangeMessage();
	void receiveExchangeMessage();
	void readShortExchangeMessage();
	void readLongExchangeMessage();

//	properties (= variables that usually should not change)
	unsigned id, fromNodeId, toNodeId;
	double freeSpeedTravelTime;
	double syncTime;
	double minimalGap;
	double squeezeTime; // the time after which an agent squeezes itself into a road
	                    // even if there is no space left for him.
	                    // this is here to avoid deadlocks in the system
	World &myWorld;
	tools::Network &myNetwork;
	Clock &myClock;


//	variables 
 	list<Agent*> startingQueue, entryQueue;
	list<Agent*> roadQueue;
 	list<double> gapQueue;
	list<SqueezeTimer> squeezeTimers;
	unsigned numGapsDirectlyAvailable;
	bool firstWaitingCarHasGap;
	bool firstWaitingCarHasSqueezeTimer;
	double lastEntryTime;
	double lastRegisterTime;
	double nextSyncTime;
	bool isInSync;
	bool directyConnected;
	RoadEnd *partnerRoadEnd;

//	message for data exchange with road end...
	vector<char> exchangeMessageSend;
	vector<char> exchangeMessageReceive;
};

class SqueezeTimer
{
public:
	SqueezeTimer(double time) :time(time), valid(true){}
	double time;
	bool valid;
};

}
}

#endif
