/***************************************************************************
 *   Copyright (C) 2005-2007 by David Charypar   *
 *   david.charypar@switzerland.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "timer.h"
#include "callable.h"
#include <iostream>

namespace allsim {
namespace microsim {


unsigned Timer::counter = 0;


Timer::Timer(Callable *callable, double time)
:theCallable(callable), time(time), sortPriority(callable->getSortPriority()), id(counter++)
{
}

int Timer::operator<(Timer const &timer) const
{
	if(!(timer.time != time || timer.sortPriority != sortPriority || timer.id == id))
	{
		cerr << "Error: timers are identical. There is no unique ordering defined!" << endl;
		cerr << "timer 1: " << timer << endl;
		cerr << "timer 2: " << *this << endl;
		throw;
	}
	if(timer.time == time) 
		return timer.sortPriority < sortPriority; 
	else 
		return timer.time < time;
}


ostream &operator<<(ostream &theStream, Timer const &timer)
{
	theStream << "callable, type, time, sortPriority, id:(" << timer.theCallable << ", " << timer.theCallable->getTypeString() 
	          << ", " << timer.time << ", " << timer.sortPriority << ", " << timer.id << ")";
	return theStream;
}


Callable *Timer::getCallable() const
{
	return theCallable;
}

double Timer::getTime() const
{
	return time;
}


}
}
