/***************************************************************************
 *   Copyright (C) 2005-2007 by David Charypar   *
 *   david.charypar@switzerland.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ALLSIMTRIP_H
#define ALLSIMTRIP_H

#include "link.h"
#include <vector>

using namespace std;

namespace allsim {
namespace microsim {


/**
@author David Charypar
*/
class Trip
{
public:
	Trip(double departureTime, double travelTime=0.0);
	~Trip();

	void addLinkIdToFrontOfRoute(unsigned newLinkId);
	void addLinkIdToBackOfRoute(unsigned newLinkId);
	void setNextLinkIndex(unsigned nextLinkIndex);
	double getDepartureTime() const;
	double getTravelTime() const;
	unsigned getNextLinkIdAndAdvance();
	unsigned getStartLinkId() const;
	unsigned getEndLinkId() const;
	unsigned getSize() const;
	bool isDone() const;
	void serializeToVector(vector<char> *theVector) const;

	static vector<char>::const_iterator unserializeFromVectorIterator(Trip*& createdTrip, vector<char>::const_iterator iter);
	static unsigned getNumCreatedTrips();

private:
	double departureTime, travelTime;
	vector<unsigned> route;
	unsigned nextLinkIndex;

	static unsigned creationCounter;

};

}
}

#endif
