/***************************************************************************
 *   Copyright (C) 2005-2007 by David Charypar   *
 *   david.charypar@switzerland.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ALLSIMWORLD_H
#define ALLSIMWORLD_H

#include "agent.h"
#include "roadstart.h"
#include "roadend.h"
#include "clock.h"
#include "network.h"

#include <map>
#include <vector>
#include <set>

using namespace std;

namespace allsim {
namespace microsim {


/**
@author David Charypar
*/
class World
{
public:
	World();
	virtual ~World();
	virtual void setNetwork(tools::Network *newNetwork);
	virtual tools::Network *getNetwork();
	virtual Clock &getClock();
	virtual void addAgent(Agent *agent);
	virtual void addRoadStart(RoadStart *roadStart);
	virtual void addRoadEnd(RoadEnd *roadEnd);
	virtual void removeAgent(unsigned agentId);
	virtual bool hasAgent(unsigned agentId);
	virtual Agent *getAgent(unsigned agentId);
	virtual void restartAgentLoop();
	virtual bool hasNextAgent();
	virtual Agent *getNextAgent();
	virtual bool hasRoadStart(unsigned roadId);
	virtual RoadStart *getRoadStart(unsigned roadId);
	virtual bool hasRoadEnd(unsigned roadId);
	virtual RoadEnd *getRoadEnd(unsigned roadId);
	virtual unsigned countAgents();
	virtual unsigned countRoadStarts();
	virtual unsigned countRoadEnds();
	virtual void directlyConnectInternalLinks();
	virtual void startAgents();
	virtual void startRoads();
	virtual double getFirstSyncTime();
	virtual void noticeAgentFinished(unsigned agentId);
	virtual unsigned getNumFinishedAgents() const;

private:
	double firstSyncTime;
	unsigned numFinishedAgents;
	Clock myClock;
	map<unsigned int, Agent *> agents;
	map<unsigned int, Agent *>::iterator currentAgentIt;
	map<unsigned int, RoadStart *> roadStarts;
	map<unsigned int, RoadEnd *> roadEnds;
	tools::Network *theNetwork;
};

}
}

#endif
