/***************************************************************************
 *   Copyright (C) 2005-2007 by David Charypar   *
 *   david.charypar@switzerland.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "eventsreader.h"
#include "event.h"

#include <iostream>
#include <fstream>

namespace allsim 
{

namespace tools 
{

EventsReader::EventsReader()
{
}


EventsReader::~EventsReader()
{
}


void EventsReader::loadEventsFromFile(char const *filename, multimap<unsigned, Event*> &eventsContainer)
{
	eventsContainer.clear();
	ifstream eventsFile(filename);
	if(!eventsFile)
	{
		cerr << "could not open events file" << endl;
		return;
	}
	unsigned eventCounter = 0;
	unsigned nextStatusOutput = 1000;
	
	while(!eventsFile.eof())
	{
		double time; 
		unsigned agentId, legNr, linkId, fromNodeId, type;
		char comment[10000];
		eventsFile >> time >> agentId >> legNr >> linkId >> fromNodeId >> type;
		eventsFile.getline(comment, 9999);
		
		if(eventCounter == nextStatusOutput)
		{
			cerr << eventCounter << endl;
			nextStatusOutput = nextStatusOutput * 11 / 10;
		}
		if(eventsFile.fail() && !eventsFile.eof())
		{
			cerr << "error in events file" << endl;
			cerr << "read: \"" << comment << "\"" << endl;
			// reset and continue
			eventsFile.clear();
			eventsFile.getline(comment, 9999);
		}
		else
		{
			// create and store event
			Event *readEvent;
// 			// do some tweeking to make events  identifyable by time
// 			if(type == 4 || type == 2)
// 				readEvent = new Event(time - 0.01, agentId, legNr, 
// 				                      linkId, fromNodeId, type);
// 			else if (type == 5 || type == 0)
// 				readEvent = new Event(time, agentId, legNr, 
// 				                      linkId, fromNodeId, type);		
// 			else if (type == 6)
// 				readEvent = new Event(time - 0.02, agentId, legNr, 
// 				                      linkId, fromNodeId, type);		

			readEvent = new Event(time, agentId, legNr, 
			                      linkId, fromNodeId, type);		

			eventsContainer.insert(pair<unsigned, Event*>(agentId, readEvent));
			eventCounter++;
		}
		
	}
	cerr << "read " << eventCounter << " events" << endl;
}


}

}
