/***************************************************************************
 *   Copyright (C) 2005-2007 by David Charypar   *
 *   david.charypar@switzerland.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "eventsstreamreader.h"
#include <iostream>

namespace allsim 
{

namespace tools 
{

EventsStreamReader::EventsStreamReader()
:currentEvent(0.0, 0, 0, 0, 0, 0),
eventsCounter(0),
nextOutputCount(100000),
hasValidEvent(false)
{
}


EventsStreamReader::~EventsStreamReader()
{
}

void EventsStreamReader::openFile(char const * eventsFilename)
{
	eventsFile.clear();	
	eventsFile.open(eventsFilename);
	if(!eventsFile)
	{
		cerr << "could not open events file" << endl;
	}
	else {
		readNextEventFromFile();
	}
}

bool EventsStreamReader::hasNextEvent()
{
	if(!hasValidEvent)
		readNextEventFromFile();
	return hasValidEvent;
}

Event EventsStreamReader::getNextEvent()
{
	if(!hasValidEvent)
	{
		readNextEventFromFile();
	}
	hasValidEvent = false;
	if(eventsCounter == nextOutputCount)
	{
		cerr << eventsCounter << endl;
		nextOutputCount = nextOutputCount * 11/10;
	}
	return currentEvent;
}

void EventsStreamReader::readNextEventFromFile()
{
//	if(!eventsFile.fail())
	{
		double time; 
		unsigned agentId, legNr, linkId, fromNodeId, type;
		char comment[10000];
		bool eventReadSuccessfully = false;
		do
		{

			eventsFile >> time >> agentId >> legNr >> linkId >> fromNodeId >> type;
			eventsFile.getline(comment, 9999);
		
			if(!eventsFile.fail())
			{
				eventReadSuccessfully = true;
			}
			else 
			if(!eventsFile.eof())
			{
				cerr << "error in events file. skipping..." << endl;
				cerr << "read: \"" << comment << "\"" << endl;
				// reset and continue
				eventsFile.clear();
				eventsFile.getline(comment, 9999);
			}
			
			
		} while (!eventReadSuccessfully && !eventsFile.fail());
		
		if(eventReadSuccessfully)
		{
			currentEvent = Event(time, agentId, legNr, 
			                      linkId, fromNodeId, type);		
			eventsCounter++;
			hasValidEvent = true;
		}
		else
		{
			hasValidEvent = false;
		}
		
	}
}


}

}
