/***************************************************************************
 *   Copyright (C) 2005-2007 by David Charypar   *
 *   david.charypar@switzerland.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ALLSIMTOOLSNETWORK_H
#define ALLSIMTOOLSNETWORK_H

#include "node.h"
#include "link.h"
#include <map>
#include <vector>

using namespace std;


namespace allsim {
namespace visualization {
class VisNetwork;
}
namespace tools {

class Network
{
public:
	/// Default constructor creates an empty network
	Network();
	/// Copy constructor. Performs a "deep" copy including nodes and links
	Network(Network const &source);
	virtual ~Network();
	
	virtual void clear();
	virtual void addNode(Node *node);
	virtual void addLink(Link *link);
	virtual Node *getNodeWithId(unsigned nodeId);
	virtual Link *getLinkWithId(unsigned linkId);
	virtual unsigned getNumNodes() const;
	virtual unsigned getNumLinks() const;
	virtual double getSpaceOnRoads();
	virtual vector<Link*> getLinksVector();

protected:
	map<unsigned int, Link*> links;
	map<unsigned int, Node*> nodes;

	friend class allsim::visualization::VisNetwork;
};

}
}

#endif
