/***************************************************************************
 *   Copyright (C) 2005-2007 by David Charypar   *
 *   david.charypar@switzerland.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "toolsglobal.h"
#include "node.h"
#include "link.h"
#include "network.h"

#include <vector>
#include <iostream>

using namespace std;

namespace allsim 
{
namespace tools
{

Node::Node(unsigned id, double xCoord, double yCoord)
:id(id), xCoord(xCoord), yCoord(yCoord)
{
//	cerr << "created Node with properties: id=" << id << ", xCoord=" << xCoord << ", yCoord=" << yCoord << endl;
}

Node::Node(Node const &source)
:id(source.id), xCoord(source.xCoord), yCoord(source.yCoord)
{
}

Node::~Node()
{
}

void Node::addOutgoingLink(Link *outgoingLink)
{
	outgoingLinks.push_back(outgoingLink);
}

void Node::addIncomingLink(Link *incomingLink)
{
	incomingLinks.push_back(incomingLink);
}


Link *Node::getLinkToNeighborNode(Node *neighborNode)
{
	for(vector<Link*>::iterator currentLink = outgoingLinks.begin(); 
	    currentLink < outgoingLinks.end(); currentLink++)
		if((*currentLink)->getToNode() == neighborNode)
			return *currentLink;
	return NULL;
}

Link *Node::getLinkFromNeighborNode(Node *neighborNode)
{
	for(vector<Link*>::iterator currentLink = incomingLinks.begin(); 
	    currentLink < incomingLinks.end(); currentLink++)
		if((*currentLink)->getFromNode() == neighborNode)
			return *currentLink;
	return NULL;
}

unsigned Node::getId() const
{
	return id;
}

double Node::getXCoord() const
{
	return xCoord;
}

double Node::getYCoord() const
{
	return yCoord;
}

unsigned Node::getNumIncomingLinks() const
{
	return incomingLinks.size();
}

unsigned Node::getNumOutgoingLinks() const
{
	return outgoingLinks.size();
}


}
}
